class SwipeActionCell < UITableViewCell

  attr_accessor :leftImage
  attr_accessor :rightImage

  attr_accessor :leftSwipeColor
  attr_accessor :rightSwipeColor

  attr_accessor :basicAlpha

  attr_accessor :swipeView

  def initWithStyle style, reuseIdentifier: reuseIdentifier
    super

    self.basicAlpha = 0

    self.leftSwipeColor  = UIColor.whiteColor
    self.rightSwipeColor = UIColor.whiteColor

    @swipeView = UIView.alloc.initWithFrame(CGRectZero).tap do |v|
      v.backgroundColor = UIColor.whiteColor
    end

    @leftImageView = UIImageView.alloc.initWithFrame(CGRectZero)
    @rightImageView = UIImageView.alloc.initWithFrame(CGRectZero)

    @scrollView = UntouchebleScrollView.alloc.initWithFrame(CGRectZero).tap do |v|
      v.showsHorizontalScrollIndicator = false
      v.delegate = self
      v.targetView = contentView

      v.addSubview @leftImageView
      v.addSubview @rightImageView
      v.addSubview @swipeView
    end

    contentView.addSubview @scrollView

    self
  end

  def leftImage= image
    @leftImageView.image = image
  end

  def rightImage= image
    @rightImageView.image = image
  end

  def scrollToContent
    point = CGPointMake(frame.size.width, 0)
    @scrollView.setContentOffset point, animated: true
  end

  def scrollViewDidEndDragging scrollView, willDecelerate: decelerate
    moveToEdges
  end

  def scrollViewDidScroll scrollView
    x = scrollView.contentOffset.x

    if x == 0 || x == frame.size.width || x == 2 * frame.size.width
      @scrollView.scrollEnabled = true
    end

    if !respond_to?(:leftSwipeAction) && x < frame.size.width
      point = CGPointMake(frame.size.width, 0)
      scrollView.setContentOffset point, animated: false
      @scrollView.scrollEnabled = true
      return
    end

    if !respond_to?(:rightSwipeAction) && x > frame.size.width
      point = CGPointMake(frame.size.width, 0)
      scrollView.setContentOffset point, animated: false
      @scrollView.scrollEnabled = true
      return
    end
    
    if x == 0
      leftSwipeAction
    elsif x == 2 * frame.size.width
      rightSwipeAction
    else
      @scrollView.backgroundColor = if x < @scrollView.frame.size.width
        alpha = basicAlpha + (1 - basicAlpha) * (@scrollView.frame.size.width - x) / @scrollView.frame.size.width

        UIColor.colorWithRed leftSwipeColor.red,
          green: leftSwipeColor.green,
          blue: leftSwipeColor.blue,
          alpha: alpha
      else
        alpha = basicAlpha + (1 - basicAlpha) * (x - @scrollView.frame.size.width) / (@scrollView.frame.size.width)

        UIColor.colorWithRed rightSwipeColor.red,
          green: rightSwipeColor.green,
          blue: rightSwipeColor.blue,
          alpha: alpha
      end
    end
  end

  def scrollViewWillBeginDecelerating scrollView
    scrollView.setContentOffset scrollView.contentOffset, animated: true
    moveToEdges
  end

  def moveToEdges
    x = @scrollView.contentOffset.x

    point = if x < @scrollView.frame.size.width / 2
      CGPointMake(0, 0)
    elsif x > @scrollView.contentSize.width / 2
      CGPointMake(2 * frame.size.width, 0)
    else
      CGPointMake(frame.size.width, 0)
    end

    @scrollView.scrollEnabled = x == frame.size.width
    @scrollView.setContentOffset point, animated: true
  end

  def layoutSubviews
    super

    size = frame.size.height - 10
    @leftImageView.frame = [[frame.size.width - size - 5, 5], [size, size]]
    @rightImageView.frame = [[2 * frame.size.width + 5, 5], [size, size]]

    @scrollView.frame = [[0, 0], [frame.size.width, frame.size.height]]
    @scrollView.contentSize = [3 * frame.size.width, frame.size.height]
    @scrollView.contentOffset = [frame.size.width, 0]

    @swipeView.frame = [[frame.size.width, 0], [frame.size.width, frame.size.height]]
  end

end